#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Strategies in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Strategies
{
	public class VPNSystem : Strategy
	{	
		private VPN VPN1;
		private bool LQD;
		private VOLMA VOLMA1;
		private VOLMA VOLMA50;
		private SMA MACLOSE;
		private RSI RSI5;
		private SMA SecondarySMA;
		private ROC SecondaryROC;
		private SMA VPNSMA;
		private ATR ATR1;
		private MAX MAX1;	
		private Series<double> VolCloseSeries;
		private SMA VolCloseSMA;
		private string errorSymbolNotFound = "";
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"The VPNSystem strategy as published in April 2021 Technical Analysis of Stocks and Commodities article “The Volume Positive Negative (VPN) Indicator Detecting High-Volume Breakouts” by Markos Katsanos.";
				Name										= "VPNSystem";
				Calculate									= Calculate.OnBarClose;
				EntriesPerDirection							= 1;
				EntryHandling								= EntryHandling.AllEntries;
				IsExitOnSessionCloseStrategy				= true;
				ExitOnSessionCloseSeconds					= 30;
				IsFillLimitOnTouch							= false;
				MaximumBarsLookBack							= MaximumBarsLookBack.TwoHundredFiftySix;
				OrderFillResolution							= OrderFillResolution.Standard;
				Slippage									= 0;
				StartBehavior								= StartBehavior.WaitUntilFlat;
				TimeInForce									= TimeInForce.Gtc;
				TraceOrders									= false;
				RealtimeErrorHandling						= RealtimeErrorHandling.StopCancelClose;
				StopTargetHandling							= StopTargetHandling.PerEntryExecution;
				BarsRequiredToTrade							= 20;
				// Disable this property for performance gains in Strategy Analyzer optimizations
				// See the Help Guide for additional information
				IsInstantiatedOnEachOptimizationIteration	= true;
				
				Period										= 30;
				Smooth										= 3;
				VPNCRIT										= 10;
				MAB											= 30;
				MinC										= 1;
				MinVol										= 100000;
				MinVC										= .5;
			}
			else if (State == State.Configure)
			{
				if (Instrument.GetInstrument("IWM", false) != null)
					AddDataSeries("IWM");
				else
					errorSymbolNotFound = "'IWM' must be added to your database.\nPlease use the Tools > Instruments menu to add this symbol, and consult your data provider for symbol mapping of 'iShares Russell 2000 Index'.\nKinetick and IQFeed may use 'IWM' and can add the symbol by opening a chart with 'IWM'";
			}
			else if(State == State.DataLoaded)
			{
				if(errorSymbolNotFound.Length != 0)
				{
					Draw.TextFixed(this, "error", errorSymbolNotFound, TextPosition.Center, Brushes.Black, new SimpleFont("Arial", 10), Brushes.Black, Brushes.White, 100);
					return;
				}
				
				VolCloseSeries 	= new Series<double>(BarsArray[0], MaximumBarsLookBack.Infinite);
			
				VPN1 			= VPN(Closes[0], Period, Smooth, VPNCRIT, MAB);		
				VOLMA1 			= VOLMA(Closes[0], 5);
				VOLMA50 		= VOLMA(Closes[0], 50);
				RSI5 			= RSI(Closes[0], 5, Smooth);
				MACLOSE 		= SMA(Closes[0], Period);		
				VolCloseSMA 	= SMA(VolCloseSeries, 5);		
				SecondarySMA 	= SMA(Closes[1], 150);
				SecondaryROC 	= ROC(Closes[1], 10);	
				VPNSMA 			= SMA(VPN1.AboveCrit, Period);
				ATR1 			= ATR(Closes[0], Period);
				MAX1 			= MAX(Closes[0], 5);
				
				AddChartIndicator(VPN1);
			}
		}

		protected override void OnBarUpdate()
		{
			if (errorSymbolNotFound.Length != 0 || BarsInProgress != 0
				|| CurrentBars[0] < Period || CurrentBars[0] < 50 || CurrentBars[1] < Period || CurrentBars[1] < 150)
				return;
		
			VolCloseSeries[0] = Volume[0] * Close[0];
			
			LQD = Close[0] > MinC && VOLMA1[0] > MinVol && VolCloseSMA[0] / Close[0] > MinVC;
			
			if(LQD && VOLMA50[0] > VOLMA50[50] && CrossAbove(VPN1.AboveCrit, VPNCRIT, 1) && RSI5[0] < 90 && Close[0] > MACLOSE[0])
				EnterLong();
			if((Closes[1][0] < SecondarySMA[0] && SecondaryROC[0] < -15)
				|| (CrossAbove(VPNSMA, VPN1.AboveCrit, 1) && Closes[0][0] < MAX1[0] - 3 * ATR1[0]))
				EnterShort();
			
			if(Position.MarketPosition == MarketPosition.Long && BarsSinceEntryExecution(0, "", 0) > ExitAfterNBars)
				ExitLong();
			
			if(Position.MarketPosition == MarketPosition.Short && BarsSinceEntryExecution(0, "", 0) > ExitAfterNBars)
				ExitShort();
		}
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Period", Order=1, GroupName="Parameters")]
		public int Period
		{ get; set; }

		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Smoothing Constant", Order=2, GroupName="Parameters")]
		public int Smooth
		{ get; set; }

		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="VPN Critical", Order=3, GroupName="Parameters")]
		public double VPNCRIT
		{ get; set; }

		[NinjaScriptProperty]
		[Range(10, int.MaxValue)]
		[Display(Name="Moving Average Bars", Description="MA Bars", Order=4, GroupName="Parameters")]
		public int MAB
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Minimum Share Price", Order=5, GroupName="Parameters")]
		public int MinC
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, int.MaxValue)]
		[Display(Name="Minimum Volume", Order=6, GroupName="Parameters")]
		public int MinVol
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="Minimum Turnover", Order=7, GroupName="Parameters")]
		public double MinVC
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, int.MaxValue)]
		[Display(Name="Exit After N Bars", Order=8, GroupName="Parameters")]
		public int ExitAfterNBars
		{ get; set; }
	}
}
